<?php

/**
 * Common function to make HTTP requests to Fourthwall
 * 
 * @param string $url The URL to fetch
 * @return array Array containing 'content' and 'error' keys
 */
function fwembed_make_request($url) {
    // Check for cached content first
    $cache_key = 'fwembed_' . md5($url);
    $cached_content = get_transient($cache_key);
    
    if ($cached_content !== false) {
        return $cached_content;
    }
    
    $options = get_option('fourthwall_settings_name');
    $ssl_verify = isset($options['ssl_verify']) ? $options['ssl_verify'] : true;
    
    $ch = curl_init();

    // More complete browser-like headers
    $headers = [
        'Accept: text/html,application/xhtml+xml,application/xml;q=0.9,image/webp,*/*;q=0.8',
        'Accept-Language: en-US,en;q=0.5',
        'Connection: keep-alive',
        'Upgrade-Insecure-Requests: 1',
        'Cache-Control: max-age=0'
    ];

    curl_setopt($ch, CURLOPT_URL, $url);
    curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
    curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
    curl_setopt($ch, CURLOPT_ENCODING, "");
    curl_setopt($ch, CURLOPT_HTTPHEADER, $headers);
    curl_setopt($ch, CURLOPT_USERAGENT, 'Mozilla/5.0 (Windows NT 10.0; Win64; x64; rv:109.0) Gecko/20100101 Firefox/115.0');
    curl_setopt($ch, CURLOPT_TIMEOUT, 30);
    curl_setopt($ch, CURLOPT_SSL_VERIFYPEER, $ssl_verify);
    curl_setopt($ch, CURLOPT_COOKIEJAR, '/tmp/cookies.txt');
    curl_setopt($ch, CURLOPT_COOKIEFILE, '/tmp/cookies.txt');

    $html_content = curl_exec($ch);
    $error = null;
    $status_code = 0;

    if (curl_errno($ch)) {
        $error = curl_error($ch);
    } else {
        $status_code = curl_getinfo($ch, CURLINFO_HTTP_CODE);
        if ($status_code == 403) {
            $error = "Access forbidden (403). The website may be blocking automated requests.";
        }
    }

    curl_close($ch);

    $result = [
        'content' => $html_content,
        'error' => $error,
        'status_code' => $status_code
    ];
    
    // Cache the result for 1 hour (3600 seconds) if no error
    if (!$error && $status_code == 200) {
        set_transient($cache_key, $result, 3600);
    }
    
    return $result;
}

/**
 * Parse HTML content and extract product tiles
 * 
 * @param string $html_content The HTML content to parse
 * @param string $base_url The base URL for constructing links
 * @return string Parsed HTML content
 */
function fwembed_parse_product_tiles($html_content, $base_url) {
    $html = null;
    libxml_use_internal_errors(true);
    $dom = new DOMDocument();
    @$dom->loadHTML(loadHTML5($html_content), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    $dom->documentURI = $base_url;
    $divs = $dom->getElementsByTagName('div');
    
    foreach ($divs as $div) {
        if ($div->hasAttribute('data-testid') && $div->getAttribute('data-testid') === 'product') {
            $xpath = new DOMXPath($dom);

            $tileLink = $xpath->query('.//a[contains(@class, "tile")]', $div);
            $tileItem = $xpath->query('.//img[contains(@class, "tile__item--1")
                                    and not(contains(@class, "badge"))
                                    and not(contains(@class, "tile_options"))
                                    and not(contains(@class, "tile__item--2"))]', $div);
            $tileDesc = $xpath->query('.//*[contains(@class, "tile__description")
                                    and not(contains(@class, "badge"))
                                    and not(contains(@class, "tile_options"))]', $div);
            
            $productHTML = '';
            $linkHref = '';
            if ($tileLink->length > 0) {
                $linkHref = $tileLink->item(0)->getAttribute('href');
            }
            if ($tileItem->length > 0) {
                $productHTML .= $dom->saveHTML($tileItem->item(0));
            }
            if ($tileDesc->length > 0) {
                $productHTML .= $dom->saveHTML($tileDesc->item(0));
            }

            $html = $html . '<div class="product-tile"><a class="product-link" target="_blank" href="' . $base_url . $linkHref . '">' . $productHTML . '</a></div>';
        }
    }
    libxml_clear_errors();
    return $html;
}

function fwembed_parse_html($url = null) {
    if ($url === null) {
        throw new ValueError("Missing URL");
    }

    $result = fwembed_make_request($url);
    
    if ($result['error']) {
        return "Error fetching URL: " . $result['error'];
    }

    return fwembed_parse_product_tiles($result['content'], $url);
}

function loadHTML5($html) {
    return '<!DOCTYPE html><html><body>' . $html . '</body></html>';
}

function fwembed_shortcode( $atts ) {
    $options = get_option( 'fourthwall_settings_name' );
    $value = isset( $options['fourth_url'] ) ? $options['fourth_url'] : 'https://fourthwall.com';
    $store_html = fwembed_parse_html($value);
    $store_render = '<div class="fw-store-parent">' . PHP_EOL . $store_html . PHP_EOL . '</div>';
    return $store_render;
}

/**
 * Parse HTML content and extract single product information
 * 
 * @param string $html_content The HTML content to parse
 * @param string $url The product URL
 * @param bool $show_description Whether to show product description
 * @return string Parsed HTML content
 */
function fwembed_parse_single_product($html_content, $url, $show_description = false) {
    $html = null;
    libxml_use_internal_errors(true);
    $dom = new DOMDocument();
    @$dom->loadHTML(loadHTML5($html_content), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    $dom->documentURI = $url;
    $xpath = new DOMXPath($dom);
    
    // Extract product information
    $productTitle = $xpath->query('//h1[@class="product-info__title"]');
    $productPrice = $xpath->query('//span[@class="product-info__price product-info__price--original"]');
    $productImage = $xpath->query('//div[@data-gallery="gallery-slide"][1]//img[@class="gallery__image-object"]');
    $productDesc = $xpath->query('//div[@class="product-info__description"]//div[@class="html-formatter"]');
    
    if ($productTitle->length > 0 && $productImage->length > 0) {
        $title = $productTitle->item(0)->textContent;
        $price = $productPrice->length > 0 ? $productPrice->item(0)->textContent : '';
        
        // Get image attributes
        $imageNode = $productImage->item(0);
        $imageSrc = $imageNode->getAttribute('src');
        $imageAlt = $imageNode->getAttribute('alt');
        
        // Build the HTML
        $html = '<div class="product-tile">';
        $html .= '<a class="product-link" target="_blank" href="' . $url . '">';
        $html .= '<img class="tile__item tile__item--1" src="' . $imageSrc . '" alt="' . htmlspecialchars($imageAlt) . '">';
        
        $html .= '<div class="tile__description">';
        $html .= '<h3 class="tile__heading">' . htmlspecialchars(trim($title)) . '</h3>';
        $html .= '<div class="tile__prices">';
        $html .= '<span class="tile__price tile__price--original">' . trim($price) . '</span>';
        $html .= '</div>';
        $html .= '</div>';
        $html .= '</a>';
        
        // Add description if show_description is true
        if ($show_description && $productDesc->length > 0) {
            $description = $dom->saveHTML($productDesc->item(0));
            $html .= '<div class="product-description">' . $description . '</div>';
        }
        
        $html .= '</div>';
    }
    
    libxml_clear_errors();
    return $html;
}

function fwembed_parse_html_single($url = null, $show_description = false) {
    if ($url === null) {
        throw new ValueError("Missing URL");
    }

    $result = fwembed_make_request($url);
    
    if ($result['error']) {
        return "Error fetching URL: " . $result['error'];
    }

    return fwembed_parse_single_product($result['content'], $url, $show_description);
}

function fwembed_single_shortcode($atts) {
    $atts = shortcode_atts(
        array(
            'url' => '',
            'show_description' => 'false',
        ),
        $atts
    );
    
    if (empty($atts['url'])) {
        return '<p>Error: URL is required for [fourthwall_single] shortcode</p>';
    }
    
    // Convert string 'true'/'false' to boolean
    $show_description = filter_var($atts['show_description'], FILTER_VALIDATE_BOOLEAN);
    
    $product_html = fwembed_parse_html_single($atts['url'], $show_description);
    return '<div class="fw-single-product">' . PHP_EOL . $product_html . PHP_EOL . '</div>';
}
add_shortcode('fourthwall_single', 'fwembed_single_shortcode');
add_shortcode( 'fourthwall', 'fwembed_shortcode' );
add_shortcode( 'fourthwall_random', 'fwembed_random_shortcode' );

/**
 * Extract all product URLs from a store page
 * 
 * @param string $html_content The HTML content to parse
 * @param string $base_url The base URL for constructing links
 * @return array Array of product URLs
 */
function fwembed_extract_product_urls($html_content, $base_url) {
    $product_urls = array();
    libxml_use_internal_errors(true);
    $dom = new DOMDocument();
    @$dom->loadHTML(loadHTML5($html_content), LIBXML_HTML_NOIMPLIED | LIBXML_HTML_NODEFDTD);
    $dom->documentURI = $base_url;
    $divs = $dom->getElementsByTagName('div');
    
    foreach ($divs as $div) {
        if ($div->hasAttribute('data-testid') && $div->getAttribute('data-testid') === 'product') {
            $xpath = new DOMXPath($dom);
            $tileLink = $xpath->query('.//a[contains(@class, "tile")]', $div);
            
            if ($tileLink->length > 0) {
                $linkHref = $tileLink->item(0)->getAttribute('href');
                $full_url = $base_url . $linkHref;
                $product_urls[] = $full_url;
            }
        }
    }
    libxml_clear_errors();
    return $product_urls;
}

/**
 * Get random products from store or specified URLs
 * 
 * @param string $store_url The store URL to fetch products from (optional if urls provided)
 * @param array $urls Array of specific product URLs to randomize from
 * @param int $count Number of products to display
 * @return string HTML content of random products
 */
function fwembed_get_random_products($store_url = null, $urls = array(), $count = 3) {
    $product_urls = array();
    
    // If specific URLs are provided, use those
    if (!empty($urls)) {
        $product_urls = $urls;
    } 
    // Otherwise, fetch all products from the store
    elseif ($store_url) {
        $result = fwembed_make_request($store_url);
        
        if ($result['error']) {
            return "Error fetching store URL: " . $result['error'];
        }
        
        $product_urls = fwembed_extract_product_urls($result['content'], $store_url);
    } else {
        return "Error: Either store URL or product URLs must be provided";
    }
    
    // Shuffle the URLs to randomize
    shuffle($product_urls);
    
    // Limit to requested count
    $selected_urls = array_slice($product_urls, 0, $count);
    
    $html = '';
    foreach ($selected_urls as $url) {
        $product_html = fwembed_parse_html_single($url, false);
        if ($product_html && !strpos($product_html, 'Error fetching URL')) {
            $html .= $product_html;
        }
    }
    
    return $html;
}

function fwembed_random_shortcode($atts) {
    $atts = shortcode_atts(
        array(
            'count' => '3',
            'urls' => '',
            'store_url' => '',
        ),
        $atts
    );
    
    $count = intval($atts['count']);
    if ($count <= 0) {
        $count = 3;
    }
    
    $urls = array();
    if (!empty($atts['urls'])) {
        // Split URLs by comma and clean them up
        $url_array = explode(',', $atts['urls']);
        foreach ($url_array as $url) {
            $clean_url = trim($url);
            if (!empty($clean_url)) {
                $urls[] = $clean_url;
            }
        }
    }
    
    $store_url = '';
    if (!empty($atts['store_url'])) {
        $store_url = trim($atts['store_url']);
    } else {
        // Use default store URL from settings if no store_url provided
        $options = get_option('fourthwall_settings_name');
        $store_url = isset($options['fourth_url']) ? $options['fourth_url'] : '';
    }
    
    $products_html = fwembed_get_random_products($store_url, $urls, $count);
    
    if (empty($products_html)) {
        return '<p>No products found to display.</p>';
    }
    
    return '<div class="fw-random-products">' . PHP_EOL . $products_html . PHP_EOL . '</div>';
}
